using System;
using System.Drawing;
using System.Drawing.Printing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
//using System.Threading;
using System.IO;
using System.IO.IsolatedStorage;
using AccellaNetApplication.Libraries.ControlsLibrary;
using System.Diagnostics;
using System.Globalization;
using System.Security.Permissions;

using Infragistics.Win.UltraWinGrid;
using Infragistics.Win;
using System.Web.Services.Protocols;

/*
 * TaskID   Date        Consultant  Description
 * 
 */

namespace AccellaNetApplication.Libraries.AccellaNetCommon
{
	public class BaseForm : System.Windows.Forms.Form 
    {
		protected AccellaNetApplication.Libraries.ControlsLibrary.SystekToolbar systekToolbar;
        protected AccellaNetApplication.Libraries.ControlsLibrary.SystekProgressBar progressBar;
        private IContainer components;

        protected System.Windows.Forms.ToolTip tooltip;

        protected DataSet ds = null;

        private SystekDataGrid[] defaultGrids = null;

        private BackgroundWorker SearchBackground;

        protected object[] searchData;


		private void EnableForm(bool enabled) 
        {
			Cursor = enabled ? Cursors.Default:Cursors.WaitCursor;
            
			systekToolbar.Enabled = enabled;            
		}

                
        public BaseForm()
        {
            InitializeComponent();

            this.systekToolbar.ButtonEvent += new AccellaNetApplication.Libraries.ControlsLibrary.SystekToolbar.ButtonClicked(this.systekToolbar_ButtonEvent);

            this.SearchBackground = new BackgroundWorker();
            this.SearchBackground.DoWork += new DoWorkEventHandler(SearchBackground_DoWork);
            this.SearchBackground.RunWorkerCompleted += new RunWorkerCompletedEventHandler(SearchBackground_RunWorkerCompleted);
        }

  

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
            this.components = new System.ComponentModel.Container();
            this.tooltip = new System.Windows.Forms.ToolTip(this.components);
            this.systekToolbar = new AccellaNetApplication.Libraries.ControlsLibrary.SystekToolbar();
            this.progressBar = new AccellaNetApplication.Libraries.ControlsLibrary.SystekProgressBar();
            this.SuspendLayout();
            // 
            // systekToolbar
            // 
            this.systekToolbar.BtnDelete = true;
            this.systekToolbar.BtnHelp = false;
            this.systekToolbar.BtnNew = true;
            this.systekToolbar.BtnPrint = true;
            this.systekToolbar.BtnRefresh = true;
            this.systekToolbar.BtnSave = true;
            this.systekToolbar.Dock = System.Windows.Forms.DockStyle.Top;
            this.systekToolbar.EnabledResetAlarm = true;
            this.systekToolbar.Location = new System.Drawing.Point(0, 0);
            this.systekToolbar.Name = "systekToolbar";
            this.systekToolbar.Size = new System.Drawing.Size(480, 30);
            this.systekToolbar.TabIndex = 9;
            this.systekToolbar.TabStop = false;
            // 
            // progressBar
            // 
            this.progressBar.BackColor = System.Drawing.SystemColors.Control;
            this.progressBar.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.progressBar.Location = new System.Drawing.Point(0, 332);
            this.progressBar.Name = "progressBar";
            this.progressBar.Size = new System.Drawing.Size(480, 21);
            this.progressBar.TabIndex = 10;
            // 
            // BaseForm
            // 
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.None;
            this.AutoScroll = true;
            this.ClientSize = new System.Drawing.Size(480, 353);
            this.Controls.Add(this.systekToolbar);
            this.Controls.Add(this.progressBar);
            this.KeyPreview = true;
            this.Name = "BaseForm";
            this.ResumeLayout(false);

		}
		#endregion

		// These should be abstract, but then the designer doesnt work
		protected virtual DataSet	LoadData() { return null; }
		protected virtual void		BindData() {}
        protected virtual bool CheckForError()
        {
            if (this.defaultGrids == null)
                return false;
            else
            {
                foreach (SystekDataGrid grid in this.defaultGrids)
                {
                    if (grid.HasError())
                        return true;
                }

                
                return false;
            }
        }

        protected virtual void systekToolbar_ButtonEvent(object sender, AccellaNetApplication.Libraries.ControlsLibrary.SystekToolbar.ToolbarClickEventArgs e) { }

        protected virtual void NewRow() { }

		

		

	        
		protected void Search() {
			Search(false);
		}

        protected DialogResult Search(bool save)
        {
			ExecuteSearch();
			return DialogResult.OK;
		}

        public bool IsReady()
        {
            return !this.SearchBackground.IsBusy;
        }

		private void ExecuteSearch()
        {
			if ( !this.Visible ) return;

            if (this.SearchBackground.IsBusy) return;

            this.SearchBackground.RunWorkerAsync();
		}
              
        private void SearchBackground_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Result != null)
            {
                if (e.Result.Equals(""))
                    BindData();
            }
        }


        private void SearchBackground_DoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = SearchWorker();
        }

		private string SearchWorker() {
            try
            {
                this.ds = LoadData();

                return "";
            }
            catch (System.Net.WebException) //TaskID: 10276, 01.06.2010, JMV - catching exception when we don't have connection to web service.
            {
                return "ERROR: Application has lost communication with web service!";
            }
            catch (SoapException soapEx) //TaskID: 10276, 01.06.2010, JMV - catching exception when we don't have connection to database
            {
                if (soapEx.Message.Contains("NoConnectionException"))
                {
                    return "ERROR: Web service has lost communication with database!";
                }
                else
                {
                    return soapEx.Message;
                }
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
		}

		protected void InitDropDown(UltraCombo combo, string band, InitializeLayoutEventArgs e, params string[] columns) 
        {
			foreach(UltraGridColumn c in e.Layout.Bands[band].Columns)
				c.Hidden = true;

			foreach(string s in columns) {
				e.Layout.Bands[band].Columns[s].Hidden = false;
                e.Layout.Bands[band].Columns[s].CellAppearance.BackColor = ColorResources.ReadOnlyColor;
			}

            if (columns.Length == 1)
            {
                e.Layout.Bands[band].ColHeadersVisible = false;
                e.Layout.Bands[band].Columns[columns[0]].PerformAutoResize(PerformAutoSizeType.AllRowsInBand);

                if (e.Layout.Bands[band].Columns[columns[0]].Width < combo.Width)
                {
                    e.Layout.Bands[band].Columns[columns[0]].Width = combo.Width;
                }
            }
		}
	}
}