using System;
using System.Drawing;

namespace AccellaNetApplication.Libraries.AccellaNetCommon {

	[Serializable]
	public class Depot {
		public string URL;
		public string DepotName;
		public Color ForeGround;
		public Color BackGround;

		public override bool Equals(object o) {
			Depot other = o as Depot;
			if ( other == null ) return false;

			return URL == other.URL;
		}

		public override int GetHashCode() {
			return URL.GetHashCode();
		}
	}

	[Serializable]
	public class DepotForm {
		public Depot Depot;
		public Type FormType;
        public string FormName;

        public DepotForm(Depot depot, Type formType)
        {
            this.Depot = depot;
            this.FormType = formType;
            this.FormName = "";
        }

		public DepotForm(Depot depot, Type formType, string formName) {
			this.Depot = depot;
			this.FormType = formType;
            this.FormName = formName;
		}

        public void SetFormName(string name)
        {
            this.FormName = name;
        }

		public override bool Equals(object o) {
			DepotForm other = o as DepotForm;
			if ( other == null ) return false;

			return Depot == other.Depot && FormType == other.FormType;
		}

		public override int GetHashCode() {
			return Depot.GetHashCode() ^ FormType.GetHashCode();
		}

		public string Caption {
			get { return Depot.DepotName + " - " + FormName; }
		}

		public string Name {
			get { return string.Format("[{0}]/[{1}]", FormType.FullName, Depot.URL); }
		}
	}

	[Serializable]
	public class FormGroup {
		private string name;
		public string Name {
			get { return name; }
			set { name = value;}
		}


		public FormGroup(string name) {
			this.Name = name;
		}
		public FormGroup() {}

		private System.Collections.ArrayList children = new System.Collections.ArrayList();

		public FormGroup AddSubGroup(string name) {
			FormGroup group = new FormGroup(name);
			children.Add(group);
			return group;
		}

		public void AddDepotForm(DepotForm form) {
			children.Add(form);
		}

		public void Clear() {
			children.Clear();
		}
	}

}