using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Threading;
using System.Globalization;

using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using AccellaNetApplication.Libraries.AccellaNetCommon;

using System.Text.RegularExpressions;

using System.Collections.Generic;

using AccellaNetApplication.Libraries.ControlsLibrary;

namespace AccellaNetApplication.Libraries.ControlsLibrary
{
	public class SystekDataGrid : Infragistics.Win.UltraWinGrid.UltraGrid
    {

		private	bool skipConfirmation = false;
		private	bool rowDeleteAllowed = true;

		private	Hashtable rowColors = new Hashtable();
		public bool	readOnlyForm = false;        
        public bool useCustomDropdownWidth;

        private string cultureSeparator = CultureInfo.CurrentCulture.NumberFormat.CurrencyDecimalSeparator;

        private string[] specialPrimaryKeys = null;
        private decimal[] specialLastActiveRow = null;

        private decimal lastActiveRow = -1m;
        private bool init = true;

        private UltraGridColumn lastActiveCellColumn = null;
        private bool initCell = true;
        

        public bool HasError()
        {
            foreach (UltraGridRow row in this.Rows)
            {
                if (row.RowSelectorAppearance.Image != null)
                    return true;
            }

            return false;
        }

		public bool SkipConfirm {
			get {return skipConfirmation;}
	 		set {skipConfirmation = value;}
		}

		public bool AllowRowDelete {
			get {return rowDeleteAllowed;}
			set {rowDeleteAllowed = value;}
		}



        private UltraDropDown dropDown1 = new UltraDropDown();
        private UltraDropDown dropDown2 = new UltraDropDown();
        protected List<int> dropDownsCount = new List<int>();

        private List<UltraDropDown> dropDownList = new List<UltraDropDown>();

		public SystekDataGrid() 
        {
            this.useCustomDropdownWidth = false;

            this.dropDownList.Add(this.dropDown1);
            this.dropDownList.Add(this.dropDown2);

            dropDownsCount.Add(0);
            dropDownsCount.Add(0);

            DisplayLayout.CaptionAppearance.FontData.Bold = DefaultableBoolean.True;
            AfterColPosChanged += new AfterColPosChangedEventHandler(this.grid_AfterColPosChanged);
            BeforeRowActivate += new RowEventHandler(this.SystekDataGrid_BeforeRowActivate);
            BeforeCellActivate += new CancelableCellEventHandler(SystekDataGrid_BeforeCellActivate);
            BeforeEnterEditMode += new CancelEventHandler(SystekDataGrid_BeforeEnterEditMode);
            AfterExitEditMode += new EventHandler(SystekDataGrid_AfterExitEditMode);
            AfterCellActivate += new EventHandler(SystekDataGrid_AfterCellActivate);
            BeforeCellDeactivate += new CancelEventHandler(SystekDataGrid_BeforeCellDeactivate);

            this.UpdateMode = Infragistics.Win.UltraWinGrid.UpdateMode.OnCellChangeOrLostFocus;
            this.DisplayLayout.Override.RowSizing = RowSizing.Fixed; // AutoFixed;            

            this.AfterRowFilterChanged += new AfterRowFilterChangedEventHandler(SystekDataGrid_AfterRowFilterChanged);
            this.FilterRow += new FilterRowEventHandler(SystekDataGrid_FilterRow);

            this.DrawFilter = new RemovePencilAndActiveRowSymbols();
            
            dropDown1.Parent = this;
            dropDown1.AfterDropDown += new DropDownEventHandler(dropDown_AfterDropDown);
            dropDown1.DisplayLayout.ScrollBounds = ScrollBounds.ScrollToFill;
            dropDown1.DisplayLayout.ScrollStyle = ScrollStyle.Immediate;

            dropDown2.Parent = this;
            dropDown2.AfterDropDown += new DropDownEventHandler(dropDown_AfterDropDown);
            dropDown2.DisplayLayout.ScrollBounds = ScrollBounds.ScrollToFill;
            dropDown2.DisplayLayout.ScrollStyle = ScrollStyle.Immediate;
        }

        // TaskID: 10439, 02.06.2010, JMV - sets the activerow to the blue-row, this is necessary if the user navigates from the filterRow
        void SystekDataGrid_BeforeCellDeactivate(object sender, CancelEventArgs e)
        {
            SetActiveRowAppearanceDefault(this.DisplayLayout);
        }

        // TaskID: 10439, 02.06.2010, JMV - created this eventhandler, so that we hide the blue-row when the user is on the filterRow, and we set the read-only
        void SystekDataGrid_AfterCellActivate(object sender, EventArgs e) // columns to yellow color when user is moving over them.
        {                                                                 // And we set the cell to be red if it has an error.
            if (this.ActiveCell.Row.IsFilterRow)
            {
                this.DisplayLayout.Override.ActiveRowAppearance.Reset();
                SetActiveCellAppearanceDefault(this.DisplayLayout);
            }
            else if (!this.ActiveCell.CanEnterEditMode)
            {
                this.DisplayLayout.Override.ActiveCellAppearance.BackColor = Color.Yellow; //Color.Orange;
            }
            else
            {
                SetActiveCellAppearanceDefault(this.DisplayLayout);
            }

            if (this.ActiveCell.Tag != null && this.ActiveCell.Tag.Equals("Error"))
            {
                this.DisplayLayout.Override.ActiveCellAppearance.BackColor = Color.Red;
            }
        }

        // TaskID: 10439, 02.06.2010, JMV - created this eventhandler, so that we can set the activeRow blue when user is finished editing cell
        void SystekDataGrid_AfterExitEditMode(object sender, EventArgs e)
        {
            DoActionsOnEditedCell();
        }

        private void DoActionsOnEditedCell()
        {
            if (this.ActiveCell.Column.Tag != null)
            {
                string tag = (string)this.ActiveCell.Column.Tag;

                if (tag.Contains("CheckUnique"))
                {
                }
                else if (tag.Contains("ExistsInBand"))
                {
                }
                else if (tag.Equals("NotNullAllowed"))
                {
                }                
            }

            if (!this.ActiveCell.Row.IsFilterRow)
            {
                SetActiveRowAppearanceDefault(this.DisplayLayout);
            }
        }



        // TaskID: 10439, 02.06.2010, JMV - created this eventhandler, so that we can "hide" the blue-row when the user is editing a cell
        void SystekDataGrid_BeforeEnterEditMode(object sender, CancelEventArgs e)
        {
            this.DisplayLayout.Override.ActiveRowAppearance.Reset();
            //this.DisplayLayout.Override.ActiveRowAppearance.BorderColor = ColorResources.ActiveRow;
        }

        private void SystekDataGrid_BeforeCellActivate(object sender, CancelableCellEventArgs e)
        {
            UltraGridCell cell = e.Cell;
            lastActiveCellColumn = cell.Column;            
        }

        private void SystekDataGrid_FilterRow(object sender, FilterRowEventArgs e)
        {
            if (e.Row.RowSelectorAppearance.Image != null)
                e.RowFilteredOut = false;
        }

        private void SystekDataGrid_AfterRowFilterChanged(object sender, AfterRowFilterChangedEventArgs e)
        {
            // ved nytt filter hopp til siste rad. hvis rad er filtrert bort hopp til frste hvis liste ikke er tom.
            foreach (UltraGridRow r in this.Rows.GetFilteredInNonGroupByRows())
            {
                DataRow dataRow = ((DataRowView)r.ListObject).Row;

                if (dataRow.Table.PrimaryKey != null && dataRow.Table.PrimaryKey.Length > 0 && (decimal)dataRow[dataRow.Table.PrimaryKey[0]] == lastActiveRow)
                {
                    r.Activate();
                    return;
                }
            }

            if (this.Rows.GetFilteredInNonGroupByRows().Length > 0)
                this.Rows.GetFilteredInNonGroupByRows()[0].Activate();
        }

		protected bool UseTestData = false;
		protected bool WriteData   = false;

		private SystekProgressBar progressBar;
		public  SystekProgressBar ProgressBar {
			get { return progressBar; }
			set { progressBar = value; }
		}

	


		//private int currentColumn = 0;
		private Hashtable nextColumnPositionForBand = new Hashtable();

		private int NextPosition(UltraGridBand band) {
			if ( !nextColumnPositionForBand.Contains(band) )
				nextColumnPositionForBand.Add(band, 0);
			int result = (int)nextColumnPositionForBand[band];
			nextColumnPositionForBand[band] = result+1;
			return result;
		}

		private decimal primaryKey = -1m;
		public decimal GetNextPrimaryKey(UltraGridRow row) {
			//return - ((DataRowView)row.ListObject).Row.Table.Rows.Count;
			return primaryKey--;
		}


        protected override void OnInitializeLayout(InitializeLayoutEventArgs e)
        {
            base.OnInitializeLayout(e);

            DisplayLayout.CaptionAppearance.FontData.Bold = DefaultableBoolean.True;
            e.Layout.CaptionAppearance.FontData.Bold = DefaultableBoolean.True;
            e.Layout.Override.HeaderClickAction = HeaderClickAction.SortSingle;
            e.Layout.Override.SelectTypeCell = SelectType.Single;
            e.Layout.Override.SelectTypeRow = SelectType.Single;
            SetActiveCellAppearanceDefault(e.Layout);
            SetActiveRowAppearanceDefault(e.Layout);

            // TaskID: 10452, 28.04.2010, JMV - added these two lines, so that the grid goes all the way down when scrolling in the scrollbar.
            e.Layout.ScrollBounds = ScrollBounds.ScrollToFill;
            e.Layout.ScrollStyle = ScrollStyle.Immediate; // fjerner tooltip'en nr man scroller
        }

        private void SetActiveRowAppearanceDefault(UltraGridLayout ultraGridLayout)
        {
            ultraGridLayout.Override.ActiveRowAppearance.BackColor = AccellaNetCommon.ColorResources.ActiveRow;
            ultraGridLayout.Override.ActiveRowAppearance.ForeColor = Color.White;
            
        }

        private void SetActiveCellAppearanceDefault(UltraGridLayout ultraGridLayout)
        {
            ultraGridLayout.Override.ActiveCellAppearance.BackColor = AccellaNetCommon.ColorResources.ActiveCell;
            ultraGridLayout.Override.ActiveCellAppearance.ForeColor = Color.Black;
        }

        public void SetFilter(Infragistics.Win.UltraWinGrid.UltraGridBand band, string[] columnsToDisable)
        {
            band.Override.FilterUIType = FilterUIType.FilterRow;
            band.Override.FilterEvaluationTrigger = FilterEvaluationTrigger.OnEnterKeyOrLeaveCell;
            band.Override.FilterOperandStyle = FilterOperandStyle.Edit;
            band.Override.FilterOperatorLocation = FilterOperatorLocation.Hidden;
            band.Override.FilterOperatorDefaultValue = FilterOperatorDefaultValue.Contains;
            band.Override.FilterClearButtonLocation = FilterClearButtonLocation.Row;
            band.Override.FilterComparisonType = FilterComparisonType.CaseInsensitive;

            foreach (UltraGridColumn col in band.Columns)
            {
                if (!col.Hidden)
                {
                    col.FilterCellAppearance.BackColor = AccellaNetCommon.ColorResources.FilterRow;
                }

                if (col.Editor is Infragistics.Win.CheckEditor)
                    col.FilterOperandStyle = FilterOperandStyle.None;
            }

            if (columnsToDisable == null) return;

            foreach (string column in columnsToDisable)
            {
                if (band.Columns.Exists(column))
                    band.Columns[column].FilterOperandStyle = FilterOperandStyle.None;
            }
        }

        protected override void OnClickCellButton(CellEventArgs e)
        {
            base.OnClickCellButton(e);

            if (e.Cell.Column.Key == "DELETE")
            {
                e.Cell.Row.Delete();
			}
		}

		protected override void OnBeforeRowsDeleted(BeforeRowsDeletedEventArgs e) {
			e.DisplayPromptMsg = false;
			base.OnBeforeRowsDeleted(e);
		}

		[Obsolete("Use InitColumn without caption instead")]
		public void InitColumn(ColumnsCollection columns, ref UltraGridColumn column, string mappingName, string caption) {
			if ( columns.IndexOf(mappingName) != -1 ) column = columns[mappingName];
			else column = columns.Add(mappingName, caption);
		}

		public void InitCheckColumn(ColumnsCollection columns, ref UltraGridColumn column, string mappingName) {
			if ( columns.IndexOf(mappingName) != -1 ) column = columns[mappingName];
			else {
				UltraGridColumn col = columns.Add();
				col.Style =  Infragistics.Win.UltraWinGrid.ColumnStyle.CheckBox;

				column = col;
			}
		}

		public void InitStringColumn(ColumnsCollection columns, ref UltraGridColumn column, string mappingName) {
			if ( columns.IndexOf(mappingName) != -1 ) column = columns[mappingName];
			else {
				UltraGridColumn col = columns.Add();
				col.Style =  Infragistics.Win.UltraWinGrid.ColumnStyle.Edit;

				column = col;
			}
		}

		public void InitColumn(ColumnsCollection columns, ref UltraGridColumn column, string mappingName) {
			if ( columns.IndexOf(mappingName) != -1 ) column = columns[mappingName];
			else column = columns.Add(mappingName);
		}

		public UltraGridColumn InitColumn(ColumnsCollection columns, string mappingName) {
			if ( columns.IndexOf(mappingName) != -1 ) return columns[mappingName];
			else return columns.Add(mappingName);
		}
		
		public UltraGridColumn ShowColumn(UltraGridColumn column, string headerCaption, int width) {
			ShowColumn(column, headerCaption);
			column.Width = width;
			return column;
		}

		public UltraGridColumn ShowColumnNoSetWritable(UltraGridColumn column, string headerCaption) {
			column.Hidden = false;
			column.Header.Caption = headerCaption;
			column.Header.VisiblePosition = NextPosition(column.Band);
			column.Header.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True;
			return column;
		}

        public UltraGridColumn ShowColumn(UltraGridColumn column, string headerCaption)
        {
            return ShowColumn(column, headerCaption, false, true, "");
        }
		public UltraGridColumn ShowColumn(UltraGridColumn column, string headerCaption, bool existsInBand, bool nullAllowed, string maskInput) {

           
			column.Hidden = false;
			column.Header.Caption = headerCaption;
			column.Header.VisiblePosition = NextPosition(column.Band);
			column.Header.Appearance.FontData.Bold = Infragistics.Win.DefaultableBoolean.True;

            if (!nullAllowed)
            {
                column.Tag = "NotNullAllowed";
            }

            if (existsInBand)
            {
                column.Tag += ",ExistsInBand";
            }

            if (!maskInput.Equals(""))
            {
                column.MaskInput = maskInput;
            }
            
			SetWritable(column, false);
			return column;
		}

		public void ShowEditColumn(UltraGridColumn column, string headerCaption) {
            ShowColumn(column, headerCaption);
            SetWritable(column, true);            
		}

        public void ShowEditColumn(UltraGridColumn column, string headerCaption, int width)
        {
            ShowEditColumn(column, headerCaption, width, true);
        }
        public void ShowEditColumn(UltraGridColumn column, string headerCaption, int width, bool nullAllowed)
        {
            ShowEditColumn(column, headerCaption, width, "", nullAllowed, false);
        }
        public void ShowEditColumn(UltraGridColumn column, string headerCaption, int width, string maskInput)
        {
            ShowEditColumn(column, headerCaption, width, maskInput, true, false);
        }
        public void ShowEditColumn(UltraGridColumn column, string headerCaption, int width, string maskInput, bool existsInBand)
        {
            ShowEditColumn(column, headerCaption, width, maskInput, true, existsInBand);
        }
		public void ShowEditColumn(UltraGridColumn column, string headerCaption, int width, string maskInput, bool nullAllowed, bool existsInBand) {
			ShowEditColumn(column, headerCaption);
			column.Width = width;
            
            if (!maskInput.Equals(""))
            {
                column.MaskInput = maskInput;                
            }

            if (!nullAllowed)
            {
                column.Tag = "NotNullAllowed";
            }
            else if (existsInBand)
            {
                column.Tag = "ExistsInBand";
            }
		}

        public UltraGridColumn ShowNumericColumn(UltraGridColumn column, string headerCaption, string format, int width, bool readOnly)
        {
            return ShowNumericColumn(column, headerCaption, format, width, readOnly, "");
        }

        public UltraGridColumn ShowNumericColumn(UltraGridColumn column, string headerCaption, string format, int width, bool readOnly, string maskInput)
        {
            return ShowNumericColumn(column, headerCaption, format, width, readOnly, maskInput, true);
        }
        public UltraGridColumn ShowNumericColumn(UltraGridColumn column, string headerCaption, string format, int width, bool readOnly, string maskInput, bool nullAllowed)
        {
            if (readOnly)
            {
                ShowColumn(column, headerCaption);
                column.Width = width;
            }
            else
            {
                ShowEditColumn(column, headerCaption, width);
            }

            column.CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;
            column.Format = format;

            // TaskID: 10352, 09.07.2010, JMV - added MaskInput, so that we only accept digits for the numeric cells
            if (maskInput != "")
            {
                column.MaskInput = maskInput;
            }

            if (!nullAllowed)
            {
                column.Tag = "NotNullAllowed";
            }

			return column;
		}

		public UltraGridColumn ShowNumericColumn(UltraGridColumn column, string headerCaption, string format) {
			ShowColumn(column, headerCaption);
			column.CellAppearance.TextHAlign = Infragistics.Win.HAlign.Right;
			column.Format = format;
			return column;
		}

		public UltraGridColumn ShowTimeColumn(UltraGridColumn column, string headerCaption, bool showDate, bool showTime) {
			return ShowTimeColumn(column, headerCaption, showDate, showTime, true, true, 0);
		}
        public UltraGridColumn ShowTimeColumn(UltraGridColumn column, string headerCaption, bool showDate, bool showTime, bool longDate, bool longTime)
        {
            return ShowTimeColumn(column, headerCaption, showDate, showTime, longDate, longTime, 0);
        }
		public UltraGridColumn ShowTimeColumn(UltraGridColumn column, string headerCaption, bool showDate, bool showTime, bool longDate, bool longTime, int width) {
			ShowColumn(column, headerCaption);
			column.Style =  Infragistics.Win.UltraWinGrid.ColumnStyle.Edit;

            CultureInfo ci = CultureInfo.CurrentCulture;
            DateTimeFormatInfo dtfi = ci.DateTimeFormat; 
                        
			string format = "";

            if (showDate)
            {
                if (longDate)
                {
                    format += dtfi.ShortDatePattern + " ";
                }
                else
                {
                    format += "ddMM ";
                }
            }

            if (showTime)
            {
                if (longTime)
                {
                    format += dtfi.LongTimePattern;
                }
                else
                {
                    format += dtfi.ShortTimePattern;
                }
            }

			column.Format = format;

            if (width == 0)
            {
                column.Width = 120;
            }
            else
            {
                column.Width = width;
            }

			column.CellAppearance.TextHAlign = HAlign.Center;
			return column;
		}

		protected DateTimeEditor ShowEditTimeColumn(UltraGridColumn column, string headerCaption, bool showDate, bool showTime) {
            return ShowEditTimeColumn(column, headerCaption, showDate, showTime, true, true);
		}
        protected DateTimeEditor ShowEditTimeColumn(UltraGridColumn column, string headerCaption, bool showDate, bool showTime, bool longDate, bool longTime)
        {
            ShowTimeColumn(column, headerCaption, showDate, showTime, longDate, longTime);
            SetWritable(column, true);
            column.CellAppearance.TextHAlign = HAlign.Center;
            column.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownCalendar;
            return (DateTimeEditor)column.Editor;
        }

        public UltraGridColumn ShowEditableNumericColumn(UltraGridColumn column, string headerCaption, string format)
        {
            return ShowEditableNumericColumn(column, headerCaption, format, "", 0, true, "");
        }
        public UltraGridColumn ShowEditableNumericColumn(UltraGridColumn column, string headerCaption, string format, string maskinput)
        {
            return ShowEditableNumericColumn(column, headerCaption, format, maskinput, 0, true, "");
        }
        public UltraGridColumn ShowEditableNumericColumn(UltraGridColumn column, string headerCaption, string format, string maskinput, bool nullAllowed, string checkType)
        {
            return ShowEditableNumericColumn(column, headerCaption, format, maskinput, 0, nullAllowed, checkType);
        }
        public UltraGridColumn ShowEditableNumericColumn(UltraGridColumn column, string headerCaption, string format, string maskinput, int width)
        {
            return ShowEditableNumericColumn(column, headerCaption, format, maskinput, width, true, "");
        }
        public UltraGridColumn ShowEditableNumericColumn(UltraGridColumn column, string headerCaption, string format, string maskinput, int width, bool nullAllowed, string checkType)
        {
            ShowNumericColumn(column, headerCaption, format);
            SetWritable(column, true);

            if (width != 0)
            {
                column.Width = width;
            }

            if (maskinput != "")
            {
                column.MaskInput = maskinput;
                column.PromptChar = ' ';
            }

            if (!nullAllowed)
            {
                column.Tag = "NotNullAllowed";               
            }

            if (!checkType.Equals(""))
            {
                if (column.Tag != null)
                {
                    column.Tag += ",CheckUnique";
                }
                else
                {
                    column.Tag = "CheckUnique";
                }
            }

            return column;
		}

		public UltraGridColumn ShowCheckBoxColumns(UltraGridColumn column, string caption) {
			ShowColumn(column, caption);
			column.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.CheckBox;
			SetWritable(column, true);

			return column;
		}






        public UltraDropDown ShowDropDownColumn(UltraGridColumn column, int index, string captionName, string display, string relationName)
        {
            return ShowDropDownColumn(column, index, captionName, display, relationName, false);
        }

		public UltraDropDown ShowDropDownColumn(UltraGridColumn column, int index, string captionName, string display, string relationName, bool isReadOnly) 
        {
			DataRelation relation = ((DataSet)DataSource).Relations[relationName];

			return ShowDropDownColumn(column, index, captionName, display, relation.ParentColumns[0].Table.TableName,
				relation.ParentColumns[0].ColumnName, isReadOnly, "");
		}

        public UltraDropDown ShowDropDownColumn(UltraGridColumn column, int index, string captionName, string display, string dataMember, string valueMember)
        {
            return ShowDropDownColumn(column, index, captionName, display, dataMember, valueMember, false, "");
        }
        public UltraDropDown ShowDropDownColumn(UltraGridColumn column, int index, string captionName, string display, string dataMember, string valueMember, string checkType) 
        {
            return ShowDropDownColumn(column, index, captionName, display, dataMember, valueMember, false, checkType);
        }
        public UltraDropDown ShowDropDownColumn(UltraGridColumn column, int index, string captionName, string display, string dataMember, string valueMember, bool isReadOnly)
        {
            return ShowDropDownColumn(column, index, captionName, display, dataMember, valueMember, isReadOnly, "");
        }
		public UltraDropDown ShowDropDownColumn(UltraGridColumn column, int index, string captionName, string display, string dataMember, string valueMember, bool isReadOnly, string checkType) {
			ShowColumn(column, captionName);

            column.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownList;
            column.ButtonDisplayStyle = Infragistics.Win.UltraWinGrid.ButtonDisplayStyle.Always;

            // TaskID: 10467, 29.04.2010, JMV - added this check, so that a regular dropdownbox is not being shown if it is read-only
            if (isReadOnly)
            {   // TaskID: 10492, 08.07.2010, JMV - had to change this function, because it didn't work with the filter-row.
                //                                  Now we only change the ButtonDisplayStyle, and not the Style of the column.
                //                                  Since the user cannot enter the cell when it is read-only, the button will never be shown.
                column.ButtonDisplayStyle = Infragistics.Win.UltraWinGrid.ButtonDisplayStyle.OnMouseEnter;
            }                       

			SetWritable(column, true);

            UltraDropDown dropDown = this.dropDownList[index];
			dropDown.SetDataBinding(this.DataSource, dataMember);
            dropDown.DisplayMember = display;
            dropDown.ValueMember = valueMember;
  
            HideAllColumns(dropDown.DisplayLayout.Bands[0].Columns);
            ShowColumn(dropDown.DisplayLayout.Bands[0].Columns[display], captionName);

            column.ValueList = dropDown;

            dropDown.DisplayLayout.Bands[0].ColHeadersVisible = false;
            
            if (!checkType.Equals(""))
            {
                column.Tag = checkType;
            }

			return dropDown;
		}

        // TaskID: 10152, 27.04.2010, JMV - Added this eventhandler to autoresize the contents of the dropdown in the grid
        void dropDown_AfterDropDown(object sender, DropDownEventArgs e)
        {
            UltraDropDown udd = sender as UltraDropDown;
            udd.DisplayLayout.PerformAutoResizeColumns(false, PerformAutoSizeType.VisibleRows);

            if (udd.DisplayLayout.Bands[0].Columns[udd.DisplayMember].Width < this.ActiveCell.Column.Width)
            {
                udd.DisplayLayout.Bands[0].Columns[udd.DisplayMember].Width = this.ActiveCell.Column.Width;
            }
        }

		public void AddRestriction(int index, UltraGridColumn restricter, UltraDropDown restricted, string restrictedCol) {
			restricted.Tag = new UltraGridColumn[] { restricter, restricted.DisplayLayout.Bands[0].Columns[restrictedCol] };

            if (IsFirstDropDown(index))
            {
                restricted.RowSelected += new RowSelectedEventHandler(restricted_RowSelected);
                restricted.BeforeDropDown += new CancelEventHandler(restricted_BeforeDropDown);
            }
        }

        private bool IsFirstDropDown(int index)
        {
            int count = this.dropDownsCount[index];
            count++;

            this.dropDownsCount[index] = count;

            if (count == 1)
                return true;
            else
                return false;
        }

        // TaskID: 10312, 07.05.2010, JMV - Geir created this new function
        public void AddRestriction(int index, UltraDropDown restricted, UltraGridColumn[] restricter, string[] restrictedCol)
        {
            UltraGridColumn[] cols = new UltraGridColumn[restricter.Length + restrictedCol.Length];

            for (int i = 0; i < restricter.Length; i++)
            {
                cols[i] = restricter[i];
                cols[restricter.Length + i] = restricted.DisplayLayout.Bands[0].Columns[restrictedCol[i]];
            }

            restricted.Tag = cols;

            if (IsFirstDropDown(index))
            {
                restricted.RowSelected += new RowSelectedEventHandler(restricted2_RowSelected);
                restricted.BeforeDropDown += new CancelEventHandler(restricted2_BeforeDropDown);
            }
        }

        void restricted_RowSelected(object sender, RowSelectedEventArgs e)
        {
            restricted_FilterDropDown(sender);
        }

		private void restricted_BeforeDropDown(object sender, CancelEventArgs e) {
            restricted_FilterDropDown(sender);
		}

        private void restricted2_RowSelected(object sender, RowSelectedEventArgs e)
        {
            restricted2_FilterDropDown(sender);
        }


		private void restricted2_BeforeDropDown(object sender, CancelEventArgs e) {
            restricted2_FilterDropDown(sender);
		}

        private void restricted_FilterDropDown(object sender)
        {
            UltraDropDown restrictedDropDown = (UltraDropDown)sender;
            UltraGridColumn restricting = ((UltraGridColumn[])restrictedDropDown.Tag)[0];
            UltraGridColumn restrictedCol = ((UltraGridColumn[])restrictedDropDown.Tag)[1];
            object restrictingValue = restricting.Band.Layout.ActiveRow.Cells[restricting].Value;


            restrictedDropDown.DisplayLayout.Bands[0].ColumnFilters[restrictedCol].FilterConditions.Clear();
            restrictedDropDown.DisplayLayout.Bands[0].ColumnFilters[restrictedCol].FilterConditions.Add(
                FilterComparisionOperator.Equals,
                restrictingValue);
        }

        // TaskID: 10312, 07.05.2010, JMV - Geir created this new function
        private void restricted2_FilterDropDown(object sender)
        {
            UltraDropDown restrictedDropDown = (UltraDropDown)sender;
            UltraGridColumn[] cols = (UltraGridColumn[])restrictedDropDown.Tag;
            UltraGridColumn[] restricting = new UltraGridColumn[cols.Length/2];
            UltraGridColumn[] restrictedCol = new UltraGridColumn[cols.Length/2];
            object[] restrictingValue = new object[cols.Length / 2];

            for (int i = 0; i < cols.Length/2; i++)
            {
                restricting[i] = cols[i];
                restrictedCol[i] = cols[i+cols.Length/2];
                restrictingValue[i] = restricting[i].Band.Layout.ActiveRow.Cells[restricting[i]].Value;
            }

            restrictedDropDown.DisplayLayout.Bands[0].ColumnFilters.ClearAllFilters();

            for (int i = 0; i < cols.Length / 2; i++)
            {
                if (restrictingValue[i] != DBNull.Value) // TaskID: 10474, 19.05.2010, JMV - this is for RoadForm to show all equipment when a PLS has not been
                {                                                                           // chosen
                    restrictedDropDown.DisplayLayout.Bands[0].ColumnFilters[restrictedCol[i]].FilterConditions.Add(
                        FilterComparisionOperator.Equals,
                        restrictingValue[i]);
                }
            }
        }

        public UltraDropDown ShowRestrictedDropDown(UltraGridColumn col, int index, string caption,
            string displayColumn, string valueColumn, string restrictionRelation)
        {
            return ShowRestrictedDropDown(col, index, caption, displayColumn, valueColumn, restrictionRelation, false);
        }

        public UltraDropDown ShowRestrictedDropDown(UltraGridColumn col, int index, string caption, 
			string displayColumn, string valueColumn, string restrictionRelation, bool isReadOnly) {

			DataRelation restriction = ((DataSet)DataSource).Relations[restrictionRelation];

			UltraDropDown dropDown = ShowDropDownColumn(col, index, caption, displayColumn, 
				restriction.ChildTable.TableName, valueColumn, isReadOnly);
			AddRestriction(index,col.Band.Columns[restriction.ParentColumns[0].ColumnName],
				dropDown, restriction.ChildColumns[0].ColumnName);

			return dropDown;
		}

		public void SortBy(UltraGridColumn col, bool descending) {
			col.Band.SortedColumns.Add(col, descending);
		}


		public void HideAllColumns(ColumnsCollection columns) {
			nextColumnPositionForBand[columns.Band] = 0;
			foreach ( UltraGridColumn column in columns ) column.Hidden = true;
		}

        public void SetWritable(UltraGridColumn column, bool canWrite)
        {
            if (readOnlyForm)
            {
                column.CellActivation = Activation.NoEdit;
                column.CellAppearance.BackColor = ColorResources.ReadOnlyColor;
            }
            else
            {
                column.CellActivation = canWrite ? Activation.AllowEdit : Activation.NoEdit;
                column.CellAppearance.BackColor = canWrite ? Color.White : ColorResources.ReadOnlyColor;
            }
            // alternative readonly color Color.LightYellow
        }

		public void SetWritable(UltraGridCell cell, bool canWrite) {
			cell.Activation = canWrite ? Activation.AllowEdit : Activation.NoEdit;
            cell.Appearance.BackColor = canWrite ? Color.White : ColorResources.ReadOnlyColor;
        }

		public void SetReadOnly(ColumnsCollection columns) {
			foreach ( UltraGridColumn column in columns ) SetWritable(column, false);
		}




        private void grid_AfterColPosChanged(object sender, AfterColPosChangedEventArgs e) {
			DisplayLayout.AutoFitStyle = AutoFitStyle.None;
		}

        

		private System.Collections.Hashtable columnTooltips = new System.Collections.Hashtable();    
		private System.Windows.Forms.ToolTip mainToolTip= new System.Windows.Forms.ToolTip();

		public void ShowOnlyColumns(UltraGridLayout layout, params string[] columnNames) 
        {
			ColumnsCollection columns = layout.Bands[0].Columns;
			foreach ( UltraGridColumn col in layout.Bands[0].Columns ) col.Hidden = true;
			foreach ( string column in columnNames ) ShowColumn(layout.Bands[0].Columns[column], column);

			//LanguageResources.SetupDropDownTexts(this,layout.Bands[0]);
		}
        		


        private bool useChild;
        private bool childSelected;

   

		private void SystekDataGrid_BeforeRowActivate(object sender, RowEventArgs e) 
        {
            UltraGridRow row = e.Row;

            if (row is UltraGridFilterRow) return;

            this.childSelected = false;

            // hvis dette er en child row som bruk parentrow som lastrow
            if (row.HasParent())
                this.childSelected = true;

            if (!this.useChild && row.HasParent())
                row = row.ParentRow;

            DataRow dataRow = ((DataRowView)row.ListObject).Row;

            if (dataRow.Table.PrimaryKey != null && dataRow.Table.PrimaryKey.Length > 0)
            {
                DataColumn pkey = dataRow.Table.PrimaryKey[0];

                if (pkey.DataType != typeof(decimal)) return;

                lastActiveRow = (decimal)dataRow[pkey];
			}
            else if (specialPrimaryKeys != null)
            {
                decimal[] temp = new decimal[specialPrimaryKeys.Length];

                for (int i = 0; i < specialPrimaryKeys.Length; i++)
                {
                    temp[i] = (decimal)dataRow[specialPrimaryKeys[i]];                    
                }

                specialLastActiveRow = temp;
            }
		}


        protected override void OnBeforeExitEditMode(Infragistics.Win.UltraWinGrid.BeforeExitEditModeEventArgs e)
        {
            base.OnBeforeExitEditMode(e);

            UltraGridCell activecell = this.ActiveCell;

            //tmmer feltet hvis man setter inn space
            if (activecell.Text == " " && !activecell.IsFilterRowCell)
            {
                activecell.SelStart = 0;
                activecell.SelLength = activecell.Column.Editor.TextLength;
                activecell.SelText = "";
            }

            //sjekker komma tegnet fr cellen oppdateres, for  unng feilmelding
            if ((activecell.Column.DataType == typeof(decimal) || activecell.Column.DataType == typeof(double))
                && activecell.Column.Editor.SupportsSelectableText && activecell.Column.Editor.IsInEditMode)
            {               
                string text = activecell.Text;
                string oldSeparator = "";
                int back;

                if (!text.Equals("") && !text.Contains(cultureSeparator))
                {
                    for (int i = 0; i < text.Length; i++)
                    {
                        if (!text[i].Equals(' '))
                        {
                            if (!int.TryParse(text[i].ToString(), out back))
                            {
                                oldSeparator = text[i].ToString();
                                break;
                            }
                        }
                    }

                    if (!oldSeparator.Equals(cultureSeparator) && !oldSeparator.Equals(""))
                    {
                        string newText = text.Replace(oldSeparator, cultureSeparator);

                        activecell.SelStart = 0;
                        activecell.SelLength = activecell.Column.Editor.TextLength;
                        activecell.SelText = newText;
                    }
                }
            }
        }

        private class RemovePencilAndActiveRowSymbols : IUIElementDrawFilter
        {
            DrawPhase IUIElementDrawFilter.GetPhasesToFilter(ref UIElementDrawParams drawParams)
            {
                if (drawParams.Element is Infragistics.Win.UltraWinGrid.RowSelectorUIElement)
                    return DrawPhase.BeforeDrawImage;
                else
                    return DrawPhase.None;
            }


            bool IUIElementDrawFilter.DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
            {
                if (drawPhase == DrawPhase.BeforeDrawImage && drawParams.Element is Infragistics.Win.UltraWinGrid.RowSelectorUIElement)
                {
                    UltraGridRow row = (UltraGridRow)drawParams.Element.GetContext(typeof(UltraGridRow));

                    if (row.IsActiveRow)
                    {
                        if (drawParams.AppearanceData.Image == null)
                        {
                            return true;
                        }
                    }
                }

                return false;
            }
        }




        private void SetPrevActiveCell(string firstVisibleColumn)
        {
            if (this.ActiveRow.Cells.Count == 0) return;

            if (lastActiveCellColumn == null && this.ActiveRow.Cells.Count > 0)
                this.ActiveCell = this.ActiveRow.Cells[firstVisibleColumn];
            else
            {
                foreach (UltraGridCell c in this.ActiveRow.Cells)
                {
                    if (c.Column.Equals(lastActiveCellColumn))
                    {
                        c.Activate();
                        return;
                    }
                }
            }
        }

        public void ActiveCellInit(string firstVisibleColumn)
        {
            if (this.ActiveRow == null) return;

            if (initCell && lastActiveCellColumn == null && this.ActiveRow.Cells.Count > 0)
            {
                this.ActiveCell = this.ActiveRow.Cells[firstVisibleColumn];
                initCell = false;
            }
            else
            {
                SetPrevActiveCell(firstVisibleColumn);
            }
        }

        public void SetSpecialPrimaryKeys(params string[] keys)
        {
            specialPrimaryKeys = keys;
        }
	}
}


